package net.rzdata.demo.dict;

import com.baomidou.mybatisplus.core.metadata.IPage;
import net.rzdata.demo.dict.domain.Dict;
import net.rzdata.demo.dict.domain.GetDictTypeListReq;
import net.rzdata.demo.dict.mapper.DictRepository;
import net.rzdata.exception.NotExistDataException;
import net.rzdata.exception.RepeatDataException;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

@Service
public class DictService {

    private final DictRepository dictRepository;

    public DictService(DictRepository dictRepository) {
        this.dictRepository = dictRepository;
    }

    protected void addDict(Dict dict) {
        check(dict);
        dictRepository.get().insert(dict);
    }

    protected List<Dict> getDictByType(String type) {
        return dictRepository.findByTypeOrderBySortAsc(type);
    }

    protected List<Dict> getDictByTypeAsTree(String type) {
        List<Dict> dictList = dictRepository.findByTypeOrderBySortAsc(type);
        Set<String> idSet = dictList.parallelStream()
            .map(Dict::getId)
            .collect(Collectors.toSet());
        List<Dict> root = dictList.parallelStream()
            .filter(dict -> !idSet.contains(dict.getParentId()))
            .toList();
        List<Dict> increment = root;
        while (!increment.isEmpty()) {
            for (Dict dict : increment) {
                List<Dict> children = dictList.parallelStream()
                    .filter(d -> Objects.equals(d.getParentId(), dict.getId()))
                    .toList();
                dict.setChildren(children);
            }
            increment = increment.parallelStream()
                .map(Dict::getChildren)
                .flatMap(List::stream)
                .toList();
        }
        return root;
    }

    protected void updateDict(Dict dict) {
        Dict exist = dictRepository.get().selectById(dict.getId());
        if (exist == null) {
            throw new NotExistDataException("字典已被删除，无法更新");
        }
        if (!Objects.equals(dict.getValue(), exist.getValue())) {
            check(dict);
        }
        dictRepository.get().updateById(dict);
        // 如果是字典类 则需要更新该类下所有值的type字段
        if (Objects.equals(exist.getParentId(), Dict.ROOT.getId())) {
            dictRepository.updateType(exist.getValue(), dict.getValue());
        }
    }

    protected IPage<Dict> getDictTypePage(GetDictTypeListReq req) {
        return dictRepository.getDictTypePage(req);
    }

    protected List<Dict> getDictTreeTopDown(Collection<String> parentId, int level) {
        if (level == 0 || level < Byte.MIN_VALUE) {
            return Collections.emptyList();
        } else if (parentId.isEmpty()) {
            return Collections.emptyList();
        }
        List<Dict> children = dictRepository.findByParentIdIn(parentId);
        Set<String> childIds = children.parallelStream()
            .map(Dict::getId)
            .collect(Collectors.toSet());
        List<Dict> grandchildren = getDictTreeTopDown(childIds, level - 1);
        for (Dict child : children) {
            List<Dict> grandchild = grandchildren.parallelStream()
                .filter(g -> Objects.equals(g.getParentId(), child.getId()))
                .toList();
            child.setChildren(grandchild);
        }
        return children;
    }

    private Optional<Dict> getDictByTypeAndValue(String type, String value) {
        List<Dict> dictList = this.getDictByType(type);
        return dictList.parallelStream()
            .filter(d -> Objects.equals(d.getValue(), value))
            .findAny();
    }

    private void check(Dict dict) {
        Optional<Dict> exist = this.getDictByTypeAndValue(dict.getType(), dict.getValue());
        if (exist.isPresent()) {
            throw new RepeatDataException(String.format("字典类型:%s, 值:%s 已存在", dict.getType(), dict.getValue()));
        }
    }

    public void deleteDict(Collection<String> ids) {
        if (ids == null || ids.isEmpty()) {
            return;
        }
        dictRepository.get().deleteBatchIds(ids);
    }
}
